package com.tuoheng.service.impl;

import com.tuoheng.mapper.AuthoritiesMapper;
import com.tuoheng.mapper.ClientUserMapper;
import com.tuoheng.mapper.ClientUserRoleMapper;
import com.tuoheng.model.dto.LoginUser;
import com.tuoheng.model.param.ClientRoleDto;
import com.tuoheng.model.param.CreateClientUserDto;
import com.tuoheng.model.param.UpdateUserClientRoleDto;
import com.tuoheng.model.param.UpdateUserPassDto;
import com.tuoheng.model.po.AuthoritiesPo;
import com.tuoheng.model.po.ClientUserRolePo;
import com.tuoheng.model.po.UserPo;
import com.tuoheng.service.ClientUserSevice;
import com.tuoheng.until.JsonResult;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.crypto.bcrypt.BCryptPasswordEncoder;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.List;

/**
 * @author chenjiandong
 * @description: TODO
 * @date 2022/10/8 11:35
 */
@Service
public class ClientUserServiceImpl implements ClientUserSevice {

    @Autowired
    private ClientUserMapper clientUserMapper;

    @Autowired
    private AuthoritiesMapper authoritiesMapper;

    @Autowired
    private ClientUserRoleMapper clientUserRoleMapper;

    @Transactional(readOnly = true)
    public JsonResult judgeCreate(String username){
        if(clientUserMapper.judgeCreateByUserName(username) > 0){
            return JsonResult.success(false);
        }
        return JsonResult.success(true);
    }

    @Transactional(rollbackFor = Exception.class)
    public JsonResult createClientUser(CreateClientUserDto createClientUserDto, LoginUser loginUser){

        if(clientUserMapper.judgeCreateByUserName(createClientUserDto.getUsername()) > 0){
            return JsonResult.error("该用户名称已存在！");
        }

        UserPo userPo = new UserPo()
                .setUsername(createClientUserDto.getUsername())
                .setPassword("{bcrypt}" + new BCryptPasswordEncoder().encode(createClientUserDto.getPassword()));
        userPo.setCreateUser(loginUser.getUserId());
        clientUserMapper.insertClientUser(userPo);

        List<AuthoritiesPo> authoritiesPos = new ArrayList<>();
        List<ClientUserRolePo> clientUserRolePoArrayList = new ArrayList<>();
        for(ClientRoleDto clientRoleDto : createClientUserDto.getClientRoleDtoList()){
            AuthoritiesPo authoritiesPo = new AuthoritiesPo()
                    .setUserId(userPo.getId())
                    .setUsername(createClientUserDto.getUsername())
                    .setAuthority(clientRoleDto.getClientId());
            authoritiesPo.setCreateUser(loginUser.getUserId());
            authoritiesPos.add(authoritiesPo);

            ClientUserRolePo clientUserRolePo = new ClientUserRolePo()
                    .setUserId(userPo.getId())
                    .setClientId(clientRoleDto.getClientId())
                    .setRoleId(clientRoleDto.getRoleId());
            clientUserRolePo.setCreateUser(loginUser.getUserId());
            clientUserRolePoArrayList.add(clientUserRolePo);

        }
        authoritiesMapper.batchInsert(authoritiesPos);
        clientUserRoleMapper.batchInsert(clientUserRolePoArrayList);
        return JsonResult.success(userPo.getId());
    }

    @Transactional(rollbackFor = Exception.class)
    public JsonResult updateUserPassword(UpdateUserPassDto updateUserPassDto, LoginUser loginUser){

        if(clientUserMapper.getUserByUserName(updateUserPassDto.getUsername()) == null){
            return JsonResult.error("该用户不存在！");
        }

        UserPo userPo = new UserPo()
                .setUsername(updateUserPassDto.getUsername())
                .setPassword("{bcrypt}" + new BCryptPasswordEncoder().encode(updateUserPassDto.getPassword()));
        userPo.setUpdateUser(loginUser.getUserId());
        clientUserMapper.updatePass(userPo);
        return JsonResult.success(true);
    }

    @Transactional(rollbackFor = Exception.class)
    public JsonResult updateUserClientRole(UpdateUserClientRoleDto updateUserClientRoleDto, LoginUser loginUser){
        UserPo userPo = clientUserMapper.getUserByUserName(updateUserClientRoleDto.getUsername());
        if(userPo == null){
            return JsonResult.error("该用户不存在！");
        }
        List<ClientRoleDto> clientRoleDtoList = updateUserClientRoleDto.getClientRoleDtoList();
        for(ClientRoleDto dto : clientRoleDtoList){
            ClientUserRolePo clientUserRolePo = new ClientUserRolePo()
                    .setUserId(userPo.getId())
                    .setClientId(dto.getClientId())
                    .setRoleId(dto.getRoleId());
            clientUserRolePo.setUpdateUser(loginUser.getUserId());
            clientUserRoleMapper.updateUserClientRole(clientUserRolePo);
        }
        return JsonResult.success(true);
    }

}
